'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _isInteger = require('babel-runtime/core-js/number/is-integer');

var _isInteger2 = _interopRequireDefault(_isInteger);

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _images = require('images');

var _images2 = _interopRequireDefault(_images);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _layout = require('layout');

var _layout2 = _interopRequireDefault(_layout);

var _message = require('./message');

var _message2 = _interopRequireDefault(_message);

var _template = require('./template');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * 合并图片、生成css
 */
var SpriteImage = function () {
  // 默认配置
  function SpriteImage() {
    var config = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
    (0, _classCallCheck3.default)(this, SpriteImage);
    this.config = {
      prefix: 'sprite',
      imgdir: '../../img',
      cssdir: '../../css',
      cssimg: '../img',
      limit: {
        width: 64000,
        height: 64000
      }
    };
    this.sort = {
      x: 'left-right',
      y: 'top-down',
      tree: 'binary-tree'
    };

    // 合并传入配置、默认配置
    this.config = (0, _extends3.default)({}, this.config, config);

    // 去掉cssimg最后的斜杠
    var cssimg = this.config.cssimg;
    if (cssimg.endsWith('/')) {
      this.config.cssimg = cssimg.substring(0, cssimg.length - 1);
    }

    // 设置处理图片的宽高限制
    _images2.default.setLimit(this.config.limit.width, this.config.limit.height);
  }

  (0, _createClass3.default)(SpriteImage, [{
    key: 'generate',
    value: function generate(dir) {
      var options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];


      if (!dir) {
        throw Error(_message2.default.PATH_NULL);
      }

      if (!_lodash2.default.isString(dir)) {
        throw Error(_message2.default.PATH_NOT_STRING);
      }

      if (dir.indexOf(_path2.default.sep) === -1) {
        throw Error(_message2.default.PATH_ERROR);
      }

      if (!_fs2.default.existsSync(dir)) {
        throw Error(_message2.default.PATH_NOT_EXISTS);
      }

      if (!_lodash2.default.isPlainObject(options)) {
        throw Error(_message2.default.CONFIG_ERROR);
      }

      options = this._handleOptions((0, _extends3.default)({ dir: dir }, options));

      this._createImage(options, this._wrapImage(options));
    }
  }, {
    key: '_handleOptions',
    value: function _handleOptions(options) {
      // 从路径中获取目录名称
      var name = options.dir.split(_path2.default.sep).pop();

      // 默认选项
      var defaults = {
        name: name,
        enable: true,
        outext: 'png',
        padding: 0,
        sort: this.sort.tree
      };

      // 如果为序列帧图，合并传入配置和默认配置
      if (options.keyframe) {
        options.keyframe = (0, _extends3.default)({
          width: 640,
          height: 1136,
          duration: 80,
          vertical: '100%',
          infinite: false
        }, options.keyframe);

        options.padding = 0;
        options.sort = this.sort.x;
      } else {
        options.sort = this.sort.hasOwnProperty(options.sort) ? this.sort[options.sort] : this.sort.tree;
      }

      return (0, _extends3.default)({}, defaults, options);
    }
  }, {
    key: '_wrapImage',
    value: function _wrapImage(options) {
      // 创建layout对象
      var layer = (0, _layout2.default)(options.sort, {
        sort: !options.keyframe
      });

      // 读取目录文件
      var imgs = _fs2.default.readdirSync(options.dir);

      // 过滤非png、jpg图片
      imgs = imgs.filter(function (file) {
        return (/\.(png|jpe?g)$/.test(file)
        );
      });

      // 序列图按照文件名称从小到大排序
      if (options.keyframe) {
        imgs.sort(function (a, b) {
          return parseInt(a) - parseInt(b);
        });
      }

      // 忽略没有图片文件的文件夹
      if (!imgs.length) {
        return;
      }

      imgs.forEach(function (file) {
        var wrap = (0, _images2.default)(_path2.default.join(options.dir, file));
        var keyframe = options.keyframe;

        // 如果为序列帧图并且图片实际宽高大于设置宽高时缩放图片
        if (keyframe && wrap.width() > keyframe.width && wrap.height() > keyframe.height) {
          wrap.resize(keyframe.width, keyframe.height);
        }

        // 添加图片到layout中
        layer.addItem({
          width: wrap.width() + options.padding,
          height: wrap.height() + options.padding,
          meta: file.replace(_path2.default.extname(file), ''),
          img: wrap
        });
      });

      // 图片排列信息
      var result = layer.export();

      if (result.items.length) {
        result.width -= options.padding;
        result.height -= options.padding;
      }

      return result;
    }
  }, {
    key: '_createImage',
    value: function _createImage(options, result) {
      // 创建合并后的图片
      var newImg = (0, _images2.default)(result.width, result.height);
      var prefix = this.config.prefix;
      var cssname = prefix + '-' + options.name + '.css';
      var imgname = prefix + '-' + options.name + '.' + options.outext;
      var cssurl = this.config.cssimg + '/' + imgname;
      var css = '';

      if (options.keyframe) {
        // 生成动画以及动画调用
        if ((0, _isInteger2.default)(options.keyframe.vertical)) {
          options.keyframe.vertical += 'px';
        }

        css += _lodash2.default.template(_template.keyframeCSS)((0, _extends3.default)({
          prefix: prefix,
          cssurl: cssurl
        }, result, options));
      } else {

        // 生成背景设置
        css += _lodash2.default.template(_template.plainBgCSS)((0, _extends3.default)({
          prefix: prefix,
          cssurl: cssurl
        }, options));
      }

      result.items.forEach(function (item) {
        // 把每张图片画到新创建的大图上
        newImg.draw(item.img, item.x, item.y);

        // 生成图片宽高位置信息
        if (!options.keyframe) {
          css += _lodash2.default.template(_template.plainItemCss)((0, _extends3.default)({
            prefix: prefix,
            item: item
          }, result, options));
        }
      });

      // 保存css
      var csspath = _path2.default.resolve(options.dir, this.config.cssdir);
      _fs2.default.existsSync(csspath) || _fs2.default.mkdirSync(csspath);
      _fs2.default.writeFileSync(_path2.default.join(csspath, cssname), css);

      // 保存图片
      var imgpath = _path2.default.resolve(options.dir, this.config.imgdir);
      _fs2.default.existsSync(imgpath) || _fs2.default.mkdirSync(imgpath);
      newImg.save(_path2.default.join(imgpath, imgname));

      // 垃圾回收
      _images2.default.gc();
    }
  }]);
  return SpriteImage;
}();

exports.default = SpriteImage;